package org.processmining.logprojection;

public interface StatisticsWithSpan extends Statistics {

	public static class Double extends Statistics.Double implements
			StatisticsWithSpan {
		private final Statistics.Double spanStats;

		public Double(double minimum, double maximum, double first, double sum,
				double sumSq, int numObservations, double span) {
			super(minimum, maximum, first, sum, sumSq, numObservations);
			spanStats = new Statistics.Double(span, span, span, span, span
					* span, 0);
		}

		public Double() {
			super();
			spanStats = new Statistics.Double();
		}

		public void addSpanObservation(double span) {
			spanStats.addObservation(span);
		}

		public double getAverageSpan() {
			return spanStats.getAverage();
		}

		public double getVarianceSpan() {
			return spanStats.getVariance();
		}

		public String toHTMLString() {
			StringBuilder builder = new StringBuilder();
			builder.append("<HTML>");
			super.fillHTMLString(builder);
			builder.append("Span:");
			spanStats.fillHTMLString(builder);
			builder.append("</HTML>");
			return builder.toString();
		}

		public int getNumObservationsSpan() {
			return spanStats.getNumObservations();
		}

		public double getMaxSpan() {
			return spanStats.getMaximum();
		}

		public double getMinSpan() {
			return spanStats.getMinimum();
		}

		public void addSpanObservation(long value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(int value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(short value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(float value) {
			spanStats.addObservation(value);
		}

	}

	public static class Long extends Statistics.Long implements
			StatisticsWithSpan {
		private final Statistics.Long spanStats;

		public Long(long minimum, long maximum, long first, long sum,
				long sumSq, int numObservations, long span) {
			super(minimum, maximum, first, sum, sumSq, numObservations);
			spanStats = new Statistics.Long(span, span, span, span,
					span * span, 0);
		}

		public Long() {
			super();
			spanStats = new Statistics.Long();
		}

		public void addSpanObservation(double span) {
			spanStats.addObservation(span);
		}

		public double getAverageSpan() {
			return spanStats.getAverage();
		}

		public double getVarianceSpan() {
			return spanStats.getVariance();
		}

		public String toHTMLString() {
			StringBuilder builder = new StringBuilder();
			builder.append("<HTML>");
			super.fillHTMLString(builder);
			builder.append("Span:");
			spanStats.fillHTMLString(builder);
			builder.append("</HTML>");
			return builder.toString();
		}

		public int getNumObservationsSpan() {
			return spanStats.getNumObservations();
		}

		public double getMaxSpan() {
			return spanStats.getMaximum();
		}

		public double getMinSpan() {
			return spanStats.getMinimum();
		}

		public void addSpanObservation(long value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(int value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(short value) {
			spanStats.addObservation(value);
		}

		public void addSpanObservation(float value) {
			spanStats.addObservation(value);
		}

	}

	public void addSpanObservation(double value);

	public void addSpanObservation(long value);

	public void addSpanObservation(int value);

	public void addSpanObservation(short value);

	public void addSpanObservation(float value);

	public double getAverageSpan();

	public double getVarianceSpan();

	public String toHTMLString();

	public int getNumObservationsSpan();
}
