package org.processmining.logprojection.filters;

import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.LogView;

public final class DiscreteRangeAttributeFilter extends AbstractEventFilter {

	private long min = Long.MIN_VALUE;
	private long max = Long.MAX_VALUE;
	private final boolean inverted;

	/**
	 * Keeps events as long as the value of the given attribute is the range
	 * [min, max] (inclusive) If inverted is set to true, then events are kept
	 * as long as the value of the given attribute is outside the range [min,
	 * max] (exclusive)
	 * 
	 * @param view
	 * @param attribute
	 *            The attribute to be checked for each event. Note that this has
	 *            to be a continuous attribute, i.e. the type should be
	 *            Attribute.DISC, Attribute.TIME_REL, or Attribute.TIME_ABS.
	 *            Otherwise no events are filtered (or all in case inverted is
	 *            true)
	 * @param isEventAttribute
	 * @param min
	 * @param max
	 * @param inverted
	 */
	public DiscreteRangeAttributeFilter(LogView view, Attribute attribute,
			long min, long max, boolean inverted) {
		super(view, attribute);
		this.inverted = inverted;
		this.min = min;
		this.max = max;
	}

	@Override
	public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
		long val;
		if (attribute.isEventAttribute()) {
			val = getValue(eventIndex);
		} else {
			val = getValue(traceIndex);
		}
		if (inverted) {
			return val < min || val > max;
		}
		return val >= min && val <= max;
	}

	private long getValue(int objectIndex) {
		switch (attribute.getType()) {
		case Attribute.DISC:
			return manager.getAttributeValueDiscrete(objectIndex,
					attributeIndex);
		case Attribute.TIME_ABS:
		case Attribute.TIME_REL:
			return manager.getAttributeValueTimestamp(objectIndex,
					attributeIndex);
		default:
			return min;
		}
	}
}