package org.processmining.logprojection.filters;

import org.processmining.logprojection.LogView;

public abstract class TraceFilter implements EventFilter {

	public static class First extends TraceFilter {

		public First(EventFilter eventFilter) {
			super(eventFilter);
		}

		@Override
		public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
			return eventFilter.keepEvent(view, traceIndex,
					view.getFirstEventNumber(traceIndex));
		}

		public boolean equals(Object o) {
			return (o instanceof First)
					&& ((First) o).eventFilter.equals(eventFilter);
		}

		public int hashCode() {
			return eventFilter.hashCode();
		}

	}

	public static class Any extends TraceFilter {

		public Any(EventFilter eventFilter) {
			super(eventFilter);
		}

		@Override
		public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
			int i = view.getFirstEventNumber(traceIndex);
			boolean found = eventFilter.keepEvent(view, traceIndex, i);
			int max = view.getFirstEventNumber(traceIndex)
					+ view.getTraceLength(traceIndex);
			for (; !found && i < max; i++) {
				found |= eventFilter.keepEvent(view, traceIndex, i);
			}
			return found;
		}

		public boolean equals(Object o) {
			return (o instanceof Any)
					&& ((Any) o).eventFilter.equals(eventFilter);
		}

		public int hashCode() {
			return eventFilter.hashCode();
		}

	}

	public static class Last extends TraceFilter {

		public Last(EventFilter eventFilter) {
			super(eventFilter);
		}

		@Override
		public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
			return eventFilter.keepEvent(
					view,
					traceIndex,
					view.getFirstEventNumber(traceIndex)
							+ view.getTraceLength(traceIndex) - 1);
		}

		public boolean equals(Object o) {
			return (o instanceof Last)
					&& ((Last) o).eventFilter.equals(eventFilter);
		}

		public int hashCode() {
			return eventFilter.hashCode();
		}

	}

	protected final EventFilter eventFilter;

	private TraceFilter(EventFilter eventFilter) {
		this.eventFilter = eventFilter;
	}
}
