package org.processmining.logprojection.lightxlog;

import java.util.Arrays;
import java.util.Iterator;

import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.LogView;
import org.processmining.logprojection.XTraceDataManager;

public class LightTrace implements Iterable<LightEvent> {

	// private final static Attribute NAMEATTRIBUTE = new Attribute("NAME",
	// Attribute.LITR, 0, false);

	private LightEvent[] events;
	private int numEvents;

	private final String name;
	private boolean carriesTraceAttributes;
	private final XTraceDataManager manager;

	public LightTrace(LogView view, String name, LightEvent... events) {
		this(view, name, false, events);
	}

	public LightTrace(LogView view, String name,
			boolean carriesTraceAttributes, LightEvent... events) {
		this.manager = view.getTraceDataManager();
		this.name = name;
		this.carriesTraceAttributes = carriesTraceAttributes;
		this.events = events;
		this.numEvents = events.length;
	}

	public LightTrace(LogView view, String name, int numEvents) {
		this(view, name, false, numEvents);
	}

	public LightTrace(LogView view, String name,
			boolean carriesTraceAttributes, int numEvents) {
		this.manager = view.getTraceDataManager();
		this.name = name;
		this.events = new LightEvent[numEvents];
		this.numEvents = 0;
	}

	public Attribute[] getAttributeKeys() {
		if (!carriesTraceAttributes) {
			// no trace attributes available other than the name.
			return new Attribute[0];
		} else {
			return manager.getKeys();
		}
	}

	public boolean getAttributeValueBoolean(Attribute key) {
		return manager
				.getAttributeValueBoolean(events[0].getTraceNumber(), key);
	}

	public boolean getAttributeValueBoolean(int attrIndex) {
		return manager.getAttributeValueBoolean(events[0].getTraceNumber(),
				attrIndex);
	}

	public long getAttributeValueDiscrete(Attribute key) {
		return manager.getAttributeValueDiscrete(events[0].getTraceNumber(),
				key);
	}

	public long getAttributeValueDiscrete(int attrIndex) {
		return manager.getAttributeValueDiscrete(events[0].getTraceNumber(),
				attrIndex);
	}

	public double getAttributeValueContinuous(Attribute key) {
		return manager.getAttributeValueContinuous(events[0].getTraceNumber(),
				key);
	}

	public double getAttributeValueContinuous(int attrIndex) {
		return manager.getAttributeValueContinuous(events[0].getTraceNumber(),
				attrIndex);
	}

	public long getAttributeValueTimestamp(Attribute key) {
		return manager.getAttributeValueTimestamp(events[0].getTraceNumber(),
				key);
	}

	public long getAttributeValueTimestamp(int attrIndex) {
		return manager.getAttributeValueTimestamp(events[0].getTraceNumber(),
				attrIndex);
	}

	public String getAttributeValueLiteral(Attribute key) {
		return manager
				.getAttributeValueLiteral(events[0].getTraceNumber(), key);
	}

	public String getAttributeValueLiteral(int attrIndex) {
		return manager.getAttributeValueLiteral(events[0].getTraceNumber(),
				attrIndex);
	}

	public int getAttributeValueLiteralIndex(Attribute key) {
		return manager.getAttributeValueLiteralIndex(
				events[0].getTraceNumber(), key);
	}

	public int getAttributeValueLiteralIndex(int attrIndex) {
		return manager.getAttributeValueLiteralIndex(
				events[0].getTraceNumber(), attrIndex);
	}

	public String getName() {
		return name;
	}

	@Override
	public Iterator<LightEvent> iterator() {
		return new Iterator<LightEvent>() {

			private int next = 0;

			@Override
			public boolean hasNext() {
				return next < events.length;
			}

			@Override
			public LightEvent next() {
				return events[next++];
			}

			@Override
			public void remove() {
				throw new UnsupportedOperationException(
						"Cannot remove events from traces");
			}

		};
	}

	public String toString() {
		return getName();
	}

	/**
	 * Equality is based on the name of the trace
	 */
	public boolean equals(Object o) {
		return (o instanceof LightTrace) && ((LightTrace) o).name.equals(name);
	}

	public int hashCode() {
		return name.hashCode();
	}

	public void addEvent(LightEvent event) {
		if (events.length == numEvents) {
			// grow the array by 16 traces
			events = Arrays.copyOf(events, numEvents + 16);
		}
		events[numEvents++] = event;
	}

	public LightEvent getEvent(int i) {
		return events[i];
	}

	public int size() {
		return events.length;
	}
}
