package org.processmining.logprojection.plugins.dottedchart.filtering;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;

import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.LogView;

public class AttributeComboBoxModel extends AbstractListModel<Attribute> implements ComboBoxModel<Attribute> {

	private static final long serialVersionUID = 314329210602187258L;
	private Attribute[] eventKeys;
	private Attribute[] traceKeys;
	private Attribute selectedObject;

	public AttributeComboBoxModel(LogView view, byte attributeType) {
		super();
		eventKeys = view.getEventDataManager().getKeysSortedAlphabetically(attributeType);
		traceKeys = view.getTraceDataManager().getKeysSortedAlphabetically(attributeType);

	}

	@Override
	public int getSize() {
		return eventKeys.length + traceKeys.length;
	}

	@Override
	public Attribute getElementAt(int index) {
		if (index < eventKeys.length) {
			return eventKeys[index];
		} else {
			return traceKeys[index - eventKeys.length];
		}
	}

	@Override
	public void setSelectedItem(Object anItem) {
		if ((selectedObject != null && selectedObject != anItem) || selectedObject == null && anItem != null) {
			selectedObject = (Attribute) anItem;
			fireContentsChanged(this, -1, -1);
		}
	}

	@Override
	public Attribute getSelectedItem() {
		return selectedObject;
	}

	public void addNone() {
		eventKeys = new Attribute[] { new Attribute("NONE", Attribute.BOOL, 0, true) };
	}

}