package org.processmining.logprojection.plugins.dottedchart.filtering;

import java.awt.BorderLayout;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.DataManager;
import org.processmining.logprojection.LogView;
import org.processmining.logprojection.filters.EventFilter;
import org.processmining.logprojection.filters.LiteralAttributeFilter;

public class LiteralAttributeFilterPanel extends
		AbstractAttributeFilterPanel<int[]> {

	private static final long serialVersionUID = 358268605631483900L;

	private int[] indices;
	private String[] values;
	private JList<String> list;
	private DefaultListModel<String> listModel;

	public LiteralAttributeFilterPanel(LogView view, Type type) {
		this(view, new HashMap<Attribute, int[]>(), type);
	}

	public LiteralAttributeFilterPanel(LogView view,
			Map<Attribute, int[]> attribute2selected, Type type) {
		super(view, Attribute.LITR, attribute2selected, type);

		listModel = new DefaultListModel<String>();
		list = new JList<String>(listModel);
		this.add(new JScrollPane(list), BorderLayout.CENTER);

		attributeComboBox.setSelectedIndex(0);
	}

	public int[] getSelectedValue() {
		int[] sel = list.getSelectedIndices();
		if (sel.length == values.length) {
			return null;
		} else {
			return sel;
		}
	}

	protected void setAttribute(Attribute a, int[] selectedIndices) {
		this.attribute = a;
		DataManager<?> manager;
		if (a.isEventAttribute()) {
			manager = view.getEventDataManager();
		} else {
			manager = view.getTraceDataManager();
		}

		indices = manager.getAttributeValueIndicesLiteral(attribute, true);
		values = manager.getAttributeValuesLiteral(attribute, true);

		listModel.removeAllElements();
		for (String s : values) {
			listModel.addElement(s);
		}
		list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		if (selectedIndices == null) {
			selectedIndices = new int[indices.length];
			for (int i = 0; i < selectedIndices.length; i++) {
				selectedIndices[i] = i;
			}
		}
		list.setSelectedIndices(selectedIndices);
		list.revalidate();
		this.revalidate();
	}

	@Override
	protected EventFilter getEventFilter(Attribute a, int[] selectedIndices) {
		if (selectedIndices == null) {
			return new LiteralAttributeFilter(view, a, new int[0]);
		}
		int[] array = Arrays.copyOf(selectedIndices, selectedIndices.length);
		DataManager<?> manager;
		if (a.isEventAttribute()) {
			manager = view.getEventDataManager();
		} else {
			manager = view.getTraceDataManager();
		}
		int[] values = manager.getAttributeValueIndicesLiteral(a, true);
		for (int i = 0; i < array.length; i++) {
			array[i] = values[array[i]];
		}
		return new LiteralAttributeFilter(view, a, array);
	}

}
