package org.processmining.logprojection.plugins.dottedchart.filtering;

import java.util.Date;
import java.util.HashMap;

import javax.swing.event.ChangeListener;

import org.jfree.chart.util.RelativeDateFormat;
import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.Histogram;
import org.processmining.logprojection.LogView;
import org.processmining.logprojection.Statistics;
import org.processmining.logprojection.filters.DiscreteRangeAttributeFilter;
import org.processmining.logprojection.filters.EventFilter;
import org.processmining.logprojection.plugins.dottedchart.filtering.AttributeFilterPanel.Type;

public class RelativeTimestampAttributeFilterPanel extends
		AbstractRangeAttributeFilterPanel<long[]> implements ChangeListener {

	private static final long serialVersionUID = 5500143718929691982L;

	private static final RelativeDateFormat formatter = new RelativeDateFormat();

	public RelativeTimestampAttributeFilterPanel(LogView view, Type type) {
		super(view, Attribute.TIME_REL, new HashMap<Attribute, long[]>(5), type);

	}

	@Override
	public long[] getSelectedValue() {
		if (minSlider.getValue() == 0
				&& maxSlider.getValue() == MAJORTICKSIZE * 10) {
			return null;
		}
		return new long[] {
				(long) (minimum + (minSlider.getValue() * conversion)),
				(long) (minimum + (maxSlider.getValue() * conversion)) };
	}

	@Override
	protected EventFilter getEventFilter(Attribute a, long[] selectedValue) {
		if (selectedValue == null) {
			return new DiscreteRangeAttributeFilter(view, a, 0, 0, false);
		} else {
			if (selectedValue[0] < selectedValue[1]) {
				return new DiscreteRangeAttributeFilter(view, a,
						selectedValue[0], selectedValue[1], false);
			} else {
				return new DiscreteRangeAttributeFilter(view, a,
						selectedValue[1], selectedValue[0], true);
			}
		}
	}

	@Override
	protected String format(double tick) {
		return formatter.format(new Date((long) tick));
	}

	@Override
	protected Statistics getStatistics(Attribute a) {
		return view.getStatisticsTimestamp(a);
	}

	@Override
	protected void setSliderValues(Attribute a, long[] selectedValue) {
		minSlider.setValue((int) ((selectedValue[0] - minimum) / conversion));
		maxSlider.setValue((int) ((selectedValue[1] - minimum) / conversion));
	}

	@Override
	protected Histogram getHistogram(Attribute a, Statistics stats, int buckets) {
		return view.getHistogramTimestamp(a, stats, buckets);
	}

}
