package org.processmining.logprojection.plugins.dottedchart.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XLog;
import org.jfree.chart.JFreeChart;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.model.ProMResource;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.DataManager;
import org.processmining.logprojection.LogView;
import org.processmining.logprojection.Statistics;
import org.processmining.logprojection.calculators.ClassifierValueCalculator;
import org.processmining.logprojection.calculators.IdenticalEventAttributeTraceCalculator;
import org.processmining.logprojection.calculators.IndexInLogAttributeCalculator;
import org.processmining.logprojection.calculators.RelativeTimeAttributeCalculator;
import org.processmining.logprojection.calculators.SequenceNumberAttributeCalculator;
import org.processmining.logprojection.calculators.StartEndTimeTraceCalculator;
import org.processmining.logprojection.plugins.dottedchart.DottedChart;
import org.processmining.logprojection.plugins.dottedchart.DottedChart.AttributeType;
import org.processmining.logprojection.plugins.dottedchart.DottedChart.DottedChartException;
import org.processmining.logprojection.plugins.dottedchart.IntensityChart;
import org.processmining.logprojection.sorting.FirstEventAttributeTraceSorter;
import org.processmining.logprojection.sorting.LastEventAttributeTraceSorter;
import org.processmining.logprojection.sorting.TraceAttributeSorter;
import org.processmining.logprojection.sorting.TraceInFileSorter;
import org.processmining.logprojection.sorting.TraceSorter;

/**
 * 
 * 
 * 
 * @author bfvdonge
 * 
 */
public class DottedChartPanel extends JPanel implements ItemListener, ChangeListener {

	private final ExecutorService executor = Executors.newCachedThreadPool();

	private static final long serialVersionUID = -2608191752875997163L;
	private LogView view;
	private DottedChart dottedChart;
	private JFreeChart chart;
	private ToolTipChartPanel chartPanel;
	private JPanel selectionPanel;
	private JComboBox<Attribute> xAxisCombo;
	private JComboBox<Attribute> yAxisCombo;

	private TraceSorter[] traceSorters;
	private JComboBox<TraceSorter> traceSortCombo;
	private JComboBox<Attribute> colorCombo;
	private JComboBox<Attribute> shapeCombo;
	private JComboBox<Attribute> statCombo;
	private JCheckBox drawLinesCheck;
	private JSlider dotSizeSlider;
	private JPanel westPanel;
	private Attribute[] eventGlobals;
	private Attribute[] traceGlobals;
	private JLabel messageLabel = new JLabel();
	private JLabel statisticsLabel = new JLabel();

	public void addListener(final PluginContext pluginContext) {
		getChartPanel().setExportListener(new ExportListener() {

			public void logExported(XLog log) {
				//TODO: Communicate with the framework to push the XLog up.
				PluginContext pc = pluginContext;
				pc.getProvidedObjectManager().createProvidedObject(XConceptExtension.instance().extractName(log), log,
						XLog.class, pc);
				if (pc instanceof UIPluginContext) {
					UIPluginContext upc = (UIPluginContext) pc;
					ProMResource<?> r = upc.getGlobalContext().getResourceManager().getResourceForInstance(log);
					r.setFavorite(true);
				}

			}
		});
	}

	public DottedChartPanel(PluginContext pluginContext, XLog l) throws DottedChartException {
		this(new LogView(l, pluginContext.getProgress()));
		addListener(pluginContext);
	}

	public DottedChartPanel(XLog l) throws DottedChartException {
		this(new LogView(l));
	}

	protected DottedChartPanel(LogView view) throws DottedChartException {
		for (XEventClassifier classifier : view.getClassifiers()) {
			ClassifierValueCalculator calculator = new ClassifierValueCalculator(classifier);
			view.getEventDataManager().addAttributes(calculator, calculator.getAttribute());
		}

		long t1 = System.currentTimeMillis();
		Attribute timestampAttribute = view.getEventDataManager().getAttributeForKey(XTimeExtension.KEY_TIMESTAMP);
		IdenticalEventAttributeTraceCalculator calculator = new IdenticalEventAttributeTraceCalculator(view);
		view.getTraceDataManager().addAttributes(calculator, calculator.getNewTraceAttributes());

		String sequenceNumber = "index in log";
		// This log uses time extension. Add attributes for time in week and
		// time in day.
		view.getTraceDataManager().addAttributes(new IndexInLogAttributeCalculator(sequenceNumber),
				new Attribute(sequenceNumber, Attribute.DISC, 0, false));

		long t2 = System.currentTimeMillis();

		if (view.getExtensions().values().contains(XTimeExtension.instance())
				&& view.getEventDataManager().containsKey(
						new Attribute(XTimeExtension.KEY_TIMESTAMP, Attribute.TIME_ABS, 0, true))) {
			String pref = XTimeExtension.instance().getPrefix();
			Attribute startTimeAttribute = new Attribute(pref + ":startTime", Attribute.TIME_ABS, 0, false);
			Attribute endTimeAttribute = new Attribute(pref + ":endTime", Attribute.TIME_ABS, 0, false);
			Attribute durationAttribute = new Attribute(pref + ":duration", Attribute.TIME_REL, 0, false);

			view.getTraceDataManager().addAttributes(
					new StartEndTimeTraceCalculator(view, timestampAttribute, startTimeAttribute.getKey(),
							endTimeAttribute.getKey(), durationAttribute.getKey()), startTimeAttribute,
					endTimeAttribute, durationAttribute);

			final String weekDay = pref + ":dayOfWeek";
			final String weekTime = pref + ":timeSinceWeekStart";
			final String dayTime = pref + ":timeSinceDayStart";
			final String caseTime = pref + ":timeSinceCaseStart";
			// This log uses time extension. Add attributes for time in week and
			// time in day.
			view.getEventDataManager().addAttributes(

					new RelativeTimeAttributeCalculator(view, weekDay, dayTime, weekTime, caseTime, timestampAttribute,
							startTimeAttribute), new Attribute(weekDay, Attribute.LITR, 0, true),
					new Attribute(weekTime, Attribute.TIME_REL, 0, true),
					new Attribute(dayTime, Attribute.TIME_REL, 0, true),
					new Attribute(caseTime, Attribute.TIME_REL, 0, true));

		}
		sequenceNumber = "index in trace";
		// This log uses time extension. Add attributes for time in week and
		// time in day.
		view.getEventDataManager().addAttributes(

		new SequenceNumberAttributeCalculator(sequenceNumber), new Attribute(sequenceNumber, Attribute.DISC, 0, true));

		doSetup(view);
	}

	private void doSetup(LogView view) throws DottedChartException {

		this.view = view;

		Attribute conceptNameAttribute = view.getTraceDataManager().getAttributeForKey(XConceptExtension.KEY_NAME);

		Attribute timestampAttribute = view.getEventDataManager().getAttributeForKey(XTimeExtension.KEY_TIMESTAMP);

		eventGlobals = view.getEventDataManager().getKeysSortedAlphabetically();
		traceGlobals = view.getTraceDataManager().getKeysSortedAlphabetically();
		Attribute[] attributes = new Attribute[eventGlobals.length + traceGlobals.length];
		for (int i = 0; i < eventGlobals.length; i++) {
			attributes[i] = eventGlobals[i];
		}
		for (int i = eventGlobals.length; i < attributes.length; i++) {
			attributes[i] = traceGlobals[i - eventGlobals.length];
		}

		if (attributes.length == 0) {
			throw new DottedChartException("No global attributes found in the log.");
		}

		if (timestampAttribute == null) {
			timestampAttribute = attributes[0];
		}
		if (conceptNameAttribute == null) {
			conceptNameAttribute = attributes[0];
		}

		JSplitPane mainPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);
		mainPanel.setResizeWeight(0.0);
		mainPanel.setOneTouchExpandable(true);

		JSplitPane centerPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false);
		centerPanel.setResizeWeight(1.0);

		dottedChart = new DottedChart(view, timestampAttribute, conceptNameAttribute, null, null);

		IntensityChart intensityChart = new IntensityChart(dottedChart, 100, 200);
		JPanel intensityPanel = new JPanel();
		intensityPanel.setUI(intensityChart);

		dottedChart.setIntensityPanel(intensityPanel, intensityChart);

		chart = new JFreeChart(dottedChart);
		chart.removeLegend();

		//SwingUtilities.windowForComponent(this)
		LegendDialog dialog = new LegendDialog(dottedChart);
		dialog.setVisible(true);

		// force aliasing of the rendered content..
		chart.getRenderingHints().put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		chartPanel = new ToolTipChartPanel(chart, dottedChart, executor);
		chartPanel.setPreferredSize(new java.awt.Dimension(1200, 600));
		chartPanel.setMinimumDrawHeight(10);
		chartPanel.setMaximumDrawHeight(2000);
		chartPanel.setMinimumDrawWidth(20);
		chartPanel.setMaximumDrawWidth(2000);

		this.setLayout(new BorderLayout());
		centerPanel.add(chartPanel, JSplitPane.TOP);
		centerPanel.add(intensityPanel, JSplitPane.BOTTOM);
		mainPanel.add(centerPanel, JSplitPane.RIGHT);

		selectionPanel = new JPanel(new GridLayout(14, 1));

		selectionPanel.add(new JLabel("X Axis Attribute"));
		xAxisCombo = new JComboBox<Attribute>(attributes);
		selectionPanel.add(xAxisCombo);
		xAxisCombo.addItemListener(this);
		xAxisCombo.setSelectedIndex(dottedChart.getXAxisAttribute().isEventAttribute() ? indexOf(eventGlobals,
				dottedChart.getXAxisAttribute()) : eventGlobals.length
				+ indexOf(traceGlobals, dottedChart.getXAxisAttribute()));

		selectionPanel.add(new JLabel("Y Axis Attribute"));
		yAxisCombo = new JComboBox<Attribute>(attributes);
		selectionPanel.add(yAxisCombo);
		yAxisCombo.addItemListener(this);
		yAxisCombo.setSelectedIndex(dottedChart.getYAxisAttribute().isEventAttribute() ? indexOf(eventGlobals,
				dottedChart.getYAxisAttribute()) : eventGlobals.length
				+ indexOf(traceGlobals, dottedChart.getYAxisAttribute()));

		selectionPanel.add(new JLabel("Trace Sorting"));
		traceSorters = new TraceSorter[2 * eventGlobals.length + traceGlobals.length + 1];
		traceSorters[0] = new TraceInFileSorter(view);
		for (int i = 0; i < eventGlobals.length; i++) {
			traceSorters[1 + 2 * i] = new FirstEventAttributeTraceSorter(view, eventGlobals[i]);
			traceSorters[2 + 2 * i] = new LastEventAttributeTraceSorter(view, eventGlobals[i]);
		}
		for (int i = 0; i < traceGlobals.length; i++) {
			traceSorters[1 + 2 * eventGlobals.length + i] = new TraceAttributeSorter(view, traceGlobals[i]);
		}

		traceSortCombo = new JComboBox<TraceSorter>(traceSorters);
		selectionPanel.add(traceSortCombo);
		traceSortCombo.setSelectedIndex(0);
		traceSortCombo.addItemListener(this);
		selectionPanel.add(new JLabel("Color Attribute"));
		colorCombo = new JComboBox<Attribute>(attributes);
		selectionPanel.add(colorCombo);
		if (dottedChart.getColorAttribute() == null) {
			colorCombo.setSelectedIndex(-1);
		} else {
			colorCombo.setSelectedIndex(dottedChart.getColorAttribute().isEventAttribute() ? indexOf(eventGlobals,
					dottedChart.getColorAttribute()) : eventGlobals.length
					+ indexOf(traceGlobals, dottedChart.getColorAttribute()));
		}
		colorCombo.addItemListener(this);
		selectionPanel.add(new JLabel("Shape Attribute"));
		shapeCombo = new JComboBox<Attribute>(attributes);
		selectionPanel.add(shapeCombo);
		if (dottedChart.getShapeAttribute() == null) {
			shapeCombo.setSelectedIndex(-1);
		} else {
			shapeCombo.setSelectedIndex(dottedChart.getShapeAttribute().isEventAttribute() ? indexOf(eventGlobals,
					dottedChart.getShapeAttribute()) : eventGlobals.length
					+ indexOf(traceGlobals, dottedChart.getShapeAttribute()));
		}
		shapeCombo.addItemListener(this);
		selectionPanel.add(new JLabel("Attribute Statistics"));
		statCombo = new JComboBox<Attribute>(attributes);
		selectionPanel.add(statCombo);
		statCombo.setSelectedIndex(-1);
		statCombo.addItemListener(this);

		drawLinesCheck = new JCheckBox("Connect events");
		drawLinesCheck.setSelected(false);
		drawLinesCheck.addItemListener(this);
		selectionPanel.add(drawLinesCheck);

		dotSizeSlider = new JSlider(0, 400, 100);
		dotSizeSlider.setMajorTickSpacing(50);
		dotSizeSlider.setPaintLabels(false);
		dotSizeSlider.setPaintTicks(true);
		dotSizeSlider.addChangeListener(this);
		selectionPanel.add(dotSizeSlider);

		westPanel = new JPanel(new BorderLayout());
		westPanel.add(selectionPanel, BorderLayout.NORTH);

		westPanel.add(statisticsLabel, BorderLayout.CENTER);

		westPanel.add(messageLabel, BorderLayout.SOUTH);
		westPanel.setMaximumSize(new Dimension(150, 150));

		//		this.add(westPanel, BorderLayout.WEST);
		mainPanel.add(westPanel, JSplitPane.LEFT);

		this.add(mainPanel, BorderLayout.CENTER);

		long t5 = System.currentTimeMillis();

		// System.out.println("Adding event attributes to traces: " + (t2 -
		// t1));
		// System.out.println("Adding relative time attributes:   " + (t3 -
		// t2));
		// System.out.println("Sorting traces   :                 " + (t4 -
		// t3));
		// System.out.println("Setting up chart :                 " + (t5 -
		// t4));
		this.invalidate();

	}

	private static <T> int indexOf(T[] array, T a) {
		if (a == null) {
			return -1;
		}
		for (int i = 0; i < array.length; i++) {
			if (array[i].equals(a)) {
				return i;
			}
		}
		return -1;
	}

	public ToolTipChartPanel getChartPanel() {
		return chartPanel;
	}

	public DottedChart getDottedChart() {
		return dottedChart;
	}

	public LogView getLogView() {
		return view;
	}

	protected void handleSorterChangeEvent() {
		final TraceSorter traceSorter = (TraceSorter) traceSortCombo.getSelectedItem();
		executor.execute(new Runnable() {

			@Override
			public void run() {
				long start = System.currentTimeMillis();
				messageLabel.setText("Sorting traces...");
				view.sortTraces(traceSorter);
				try {
					dottedChart.setXAxisAttribute(dottedChart.getXAxisAttribute(), false);
					dottedChart.setYAxisAttribute(dottedChart.getYAxisAttribute(), false);
					dottedChart.setColorAttribute(dottedChart.getColorAttribute(), false);
					dottedChart.setShapeAttribute(dottedChart.getShapeAttribute(), true);
					long end = System.currentTimeMillis();
					messageLabel.setText("Traces sorted.(in " + (end - start) + " ms)");
				} catch (DottedChartException e1) {
					statisticsLabel.setText(e1.getMessage());
					messageLabel.setText("Error while sorting traces.");
				}
			}
		});

	}

	protected void handleAttributeChangeEvent(final JComboBox<Attribute> combo, final AttributeType type,
			final String name) {
		final Attribute oldAttribute = dottedChart.getAttribute(type);
		final Attribute newAttribute = (Attribute) combo.getSelectedItem();
		if (newAttribute == oldAttribute) {
			return;
		}
		executor.execute(new Runnable() {

			@Override
			public void run() {
				long start = System.currentTimeMillis();
				messageLabel.setText("Updating " + name + "...");
				try {
					dottedChart.setAttribute(type, newAttribute, true);
					long end = System.currentTimeMillis();
					messageLabel.setText(name + " updated in " + (end - start) + " ms.");
				} catch (DottedChartException e1) {
					statisticsLabel.setText(e1.getMessage());
					messageLabel.setText("Error (reverting)...");
					try {
						dottedChart.setAttribute(type, oldAttribute, true);
						messageLabel.setText(name + " not changed.");
						combo.setSelectedItem(oldAttribute);
					} catch (DottedChartException e2) {
						statisticsLabel.setText(e2.getMessage());
						messageLabel.setText("Error while changing " + name + ".");
					}
				}
			}
		});
	}

	protected void handleStatisticsChange() {
		int idx = statCombo.getSelectedIndex();
		final Attribute a;
		final DataManager<?> manager;
		if (idx < eventGlobals.length) {
			a = eventGlobals[idx];
			manager = view.getEventDataManager();
		} else {
			a = traceGlobals[idx - eventGlobals.length];
			manager = view.getTraceDataManager();
		}
		executor.execute(new Runnable() {

			@Override
			public void run() {
				long start = System.currentTimeMillis();
				String description;
				messageLabel.setText("Computing statistics...");
				if (a.getType() == Attribute.LITR) {
					int count = view.getDifferentValueCount(a);
					description = "<html>No statistics available for<BR>" + "literal attribute " + a.getKey() + "<BR>"
							+ "Value count is " + count + ".</html>";
				} else if (a.getType() == Attribute.BOOL) {
					description = "<html>No statistics available for<BR>" + "boolean attribute " + a.getKey()
							+ ".</html>";
				} else if (a.getType() == Attribute.CONT) {
					Statistics stats = view.getStatisticsContinuous(a);
					description = stats.toHTMLString();
				} else if (a.getType() == Attribute.DISC) {
					Statistics stats = view.getStatisticsDiscrete(a);
					description = stats.toHTMLString();
				} else if (a.getType() == Attribute.TIME_ABS || (a.getType() == Attribute.TIME_REL)) {
					Statistics stats = view.getStatisticsTimestamp(a);
					description = stats.toHTMLString();
				} else {
					description = "<html>No statistics available for attribute<BR>" + a.getKey()
							+ " of unknown type.</html>";
				}
				statisticsLabel.setText(description);
				long end = System.currentTimeMillis();
				messageLabel.setText("Statistics computes. (in " + (end - start) + " ms)");
			}
		});
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == drawLinesCheck) {
			dottedChart.setLinesOn(drawLinesCheck.isSelected(), true);
			return;
		}
		if (e.getStateChange() != ItemEvent.SELECTED) {
			return;
		}
		if (e.getSource() == shapeCombo) {
			handleAttributeChangeEvent(shapeCombo, AttributeType.SHAPE, "shapes");
		} else if (e.getSource() == colorCombo) {
			handleAttributeChangeEvent(colorCombo, AttributeType.COLOR, "colors");
		} else if (e.getSource() == xAxisCombo) {
			handleAttributeChangeEvent(xAxisCombo, AttributeType.XAXIS, "x-axis");
		} else if (e.getSource() == yAxisCombo) {
			handleAttributeChangeEvent(yAxisCombo, AttributeType.YAXIS, "y-axis");
		} else if (e.getSource() == traceSortCombo) {
			handleSorterChangeEvent();
		} else if (e.getSource() == statCombo) {
			handleStatisticsChange();
		}
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		assert e.getSource() == dotSizeSlider;
		double size = dotSizeSlider.getValue() / 100.0;
		if (!dotSizeSlider.getValueIsAdjusting()) {
			if (size == 0) {
				size = 1;
			}
			dottedChart.setDotScale(size, true);
		}
	}
}
